import requests
import copy

try:
    import urlparse
except ImportError:
    import urllib.parse as urlparse

from .auth_handler import AuthHandler
from .error_handler import ErrorHandler
from .request_handler import RequestHandler
from .response import Response
from .response_handler import ResponseHandler


class HttpClient(object):

    """Main HttpClient which is used by API classes"""

    def __init__(self, auth, options):

        if isinstance(auth, str):
            auth = {'http_header': auth}

        self.options = {
            'base': 'https://api.quickemailverification.com',
            'api_version': 'v1',
            'user_agent': 'quickemailverification-python/1.0.4 (https://github.com/quickemailverification/quickemailverification-python)'
        }

        self.options.update(options)

        self.base = self.options['base']

        self.headers = {
            'user-agent': self.options['user_agent']
        }

        if 'headers' in self.options:
            self.headers.update(self.dict_key_lower(self.options['headers']))
            del self.options['headers']

        self.auth = AuthHandler(auth)

    def get(self, path, params={}, options={}):
        options.update({'query': params})
        return self.request(path, None, 'get', options)

    def post(self, path, body={}, options={}):
        return self.request(path, body, 'post', options)

    def patch(self, path, body={}, options={}):
        return self.request(path, body, 'patch', options)

    def delete(self, path, body={}, options={}):
        return self.request(path, body, 'delete', options)

    def put(self, path, body={}, options={}):
        return self.request(path, body, 'put', options)

    def request(self, path, body, method, options):
        """Intermediate function which does three main things

        - Transforms the body of request into correct format
        - Creates the requests with given parameters
        - Returns response body after parsing it into correct format
        """
        kwargs = copy.deepcopy(self.options)
        kwargs.update(options)

        kwargs['headers'] = copy.deepcopy(self.headers)

        if 'headers' in options:
            kwargs['headers'].update(self.dict_key_lower(options['headers']))

        kwargs['data'] = body
        kwargs['allow_redirects'] = True

        kwargs['params'] = kwargs['query'] if 'query' in kwargs else {}

        if 'query' in kwargs:
            del kwargs['query']

        if 'body' in kwargs:
            del kwargs['body']

        del kwargs['base']
        del kwargs['user_agent']

        if method != 'get':
            kwargs = self.set_body(kwargs)

        kwargs['hooks'] = dict(response=ErrorHandler.check_error)

        kwargs = self.auth.set(kwargs)

        response = self.create_request(method, path, kwargs)

        return Response(
            self.get_body(response), response.status_code, response.headers
        )

    def create_request(self, method, path, options):
        """Creating a request with the given arguments

        If api_version is set, appends it immediately after host
        """
        version = '/' + options['api_version'] if 'api_version' in options else ''

        path = urlparse.urljoin(self.base, version + path)

        if 'api_version' in options:
            del options['api_version']

        if 'response_type' in options:
            del options['response_type']

        return requests.request(method, path, **options)

    def get_body(self, response):
        """Get response body in correct format"""
        return ResponseHandler.get_body(response)

    def set_body(self, request):
        """Set request body in correct format"""
        return RequestHandler.set_body(request)

    def dict_key_lower(self, dic):
        """Make dict keys all lower case"""
        return dict(zip(map(self.key_lower, dic.keys()), dic.values()))

    def key_lower(self, key):
        """Make a function for lower case"""
        return key.lower()



from .http_client import HttpClient

# Assign all the api classes
from .api.quickemailverification import Quickemailverification


class Client(object):

    def __init__(self, auth={}, options={}):
        self.http_client = HttpClient(auth, options)

    def quickemailverification(self):
        """QuickEmailVerification Class for email verification
        """
        return Quickemailverification(self.http_client)




from ..error import ClientError
from .response_handler import ResponseHandler


class ErrorHandler(object):

    """ErrorHandler takes care of getting the error message from response body"""

    @staticmethod
    def check_error(response, *args, **kwargs):
        code = response.status_code
        typ = response.headers.get('content-type')

        if code in range(500, 600):
            raise ClientError('Error ' + str(code), code)
        elif code in range(400, 500):
            body = ResponseHandler.get_body(response)
            arrErrorCode = [400, 401, 402, 403, 404, 429]

            if code not in arrErrorCode:
                # If HTML, whole body is taken
                if isinstance(body, str):
                    message = body

                # If JSON, a particular field is taken and used
                if typ.find('json') != -1 and isinstance(body, dict):
                    if 'message' in body:
                        message = body['message']
                    else:
                        message = 'Unable to select error message from json returned by request responsible for error'

                if message == '':
                    message = 'Unable to understand the content type of response returned by request responsible for error'

                raise ClientError(message, code)



from codecs import open
from os import path
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

def read(*parts):
    # intentionally *not* adding an encoding option to open
    # see here: https://github.com/pypa/virtualenv/issues/201#issuecomment-3145690
    return open(path.join(here, *parts), 'r').read()

setup(
    name='quickemailverification',
    version='1.0.4',
    description='Official QuickEmailVerification API library client for python',
    long_description=long_description,
    author='QuickEmailVerification',
    author_email='support@quickemailverification.com',
    url='https://github.com/quickemailverification/quickemailverification-python',
    license='MIT',
    install_requires=[
        'requests >= 2.1.0',
        'six >= 1.9.0'
    ],
    packages=[
        'quickemailverification',
        'quickemailverification.api',
        'quickemailverification.error',
        'quickemailverification.http_client'
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)
